<?php
/***********************************************************************
Copyright (C) 2012 Matyas Danter

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"), 
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom the 
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES 
OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
OTHER DEALINGS IN THE SOFTWARE.
*************************************************************************/

/**
 * This suite tests point validity, x9.x.x etc. for EC, ECDSA,ECDH
 * This class also demonstrates how to use the EC classes.
 *
 * @author Matej Danter
 */
class TestSuite {

    private $START_PRIME = 31;
    private $NUM_PRIMES = 10;

    public function __construct($verbose = false) {
        if (extension_loaded('gmp') && USE_EXT == 'GMP') {
            $start_time = microtime(true);
            echo "GMP PHP extension was found and is preferred for performance reasons.<br /><b>Initiating tests.</b><br />\n";
            if (!$verbose) {
                echo "You selected NON-VERBOSE mode. <br /><b>ONLY FAILURES and TIME STATISTICS are REPORTED.</b><br />\n";
            } else {
                echo "You selected VERBOSE mode. <br /><b>ALL OUTCOMES are REPORTED.</b><br />\n";
            }
            echo "<br /><br />--------- START NEXT PRIME TEST ---------<br /><br />\n";

            self::gmp_NextPrime($this->START_PRIME, $this->NUM_PRIMES, $verbose);
            echo "<br /><br />--------- END NEXT PRIME TEST ---------<br /><br />\n";
            echo "--------- START SQUARE ROOT MOD P TEST ---------<br /><br />\n";

            self::gmp_squareRootModP($this->START_PRIME, $verbose);
            echo "<br /><br />--------- END SQUARE ROOT MOD P TEST ---------<br /><br />\n";
            echo "--------- START MULTIPLICATIVE INVERSE MOD P TEST ---------<br /><br />\n";

            self::gmp_multInverseModP($verbose);
            echo "<br /><br />--------- END MULTIPLICATIVE INVERSE TEST ---------<br /><br />\n";
            echo "--------- START ELLIPTIC CURVE ARITHMETIC TEST ---------<br /><br />\n";

            self::gmp_EcArithmetic($verbose);
            echo "<br /><br />--------- END ELLIPTIC CURVE ARITHMETIC TEST ---------<br /><br />\n";
            echo "--------- START NIST PUBLISHED CURVES TEST ---------<br /><br />\n";

            self::gmp_NISTCurveTest($verbose);
            echo "<br /><br />--------- END NIST PUBLISHED CURVES TEST ---------<br /><br />\n";
            echo "--------- START POINT VALIDITY TEST ---------<br /><br />\n";
            self::gmp_pointValidity($verbose);
            echo "<br /><br />--------- END POINT VALIDITY TEST ---------<br /><br />\n";
            echo "--------- START SIGNATURE VALIDITY TEST ---------<br /><br />\n";
            self::gmp_signatureValidity($verbose);
            echo "<br /><br />--------- END SIGNATURE VALIDITY TEST ---------<br /><br />\n";
            echo "--------- START DIFFIE HELLMAN KEY EXCHANGE TEST ---------<br /><br />\n";
            self::gmp_diffieHellman($verbose);
            echo "<br /><br />--------- END DIFFIE HELLMAN KEY EXHANGE TEST ---------<br /><br />\n";
            $end_time = microtime(true);

            $time_res = $end_time - $start_time;

            echo "<br /><h3>TEST SUITE TOTAL TIME : " . $time_res . " seconds. </h3><br />";
        } else if (extension_loaded('bcmath') && USE_EXT == 'BCMATH') {
            $start_time = microtime(true);
            echo "<b>BCMATH PHP extension</b> was found performance will tend to <b>SEVERELY LACK USABILITY</b>. Consider installing GMP. <br /><b>Initiating tests.</b><br />\n";
            if (!$verbose) {
                echo "You selected NONE-VERBOSE mode. <br /><b>ONLY FAILURES and TIME STATISTICS are REPORTED.</b><br />\n";
            } else {
                echo "You selected VERBOSE mode. <br /><b>ALL OUTCOMES are REPORTED.</b><br />\n";
            }
            echo "<br /><br />--------- START NEXT PRIME TEST ---------<br /><br />\n";

            self::bcmath_NextPrime($this->START_PRIME, $this->NUM_PRIMES, $verbose);
            echo "<br /><br />--------- END NEXT PRIME TEST ---------<br /><br />\n";
            echo "--------- START SQUARE ROOT MOD P TEST ---------<br /><br />\n";

            self::bcmath_squareRootModP($this->START_PRIME, $verbose);
            echo "<br /><br />--------- END SQUARE ROOT MOD P TEST ---------<br /><br />\n";
            echo "--------- START MULTIPLICATIVE INVERSE MOD P TEST ---------<br /><br />\n";

            self::bcmath_multInverseModP($verbose);
            echo "<br /><br />--------- END MULTIPLICATIVE INVERSE TEST ---------<br /><br />\n";
            echo "--------- START ELLIPTIC CURVE ARITHMETIC TEST ---------<br /><br />\n";

            self::bcmath_EcArithmetic($verbose);
            echo "<br /><br />--------- END ELLIPTIC CURVE ARITHMETIC TEST ---------<br /><br />\n";
            echo "--------- START NIST PUBLISHED CURVES TEST ---------<br /><br />\n";

            self::bcmath_NISTCurveTest($verbose);
            echo "<br /><br />--------- END NIST PUBLISHED CURVES TEST ---------<br /><br />\n";
            echo "--------- START POINT VALIDITY TEST ---------<br /><br />\n";
            self::bcmath_pointValidity($verbose);
            echo "<br /><br />--------- END POINT VALIDITY TEST ---------<br /><br />\n";
            echo "--------- START SIGNATURE VALIDITY TEST ---------<br /><br />\n";
            self::bcmath_signatureValidity($verbose);
            echo "<br /><br />--------- END SIGNATURE VALIDITY TEST ---------<br /><br />\n";
            echo "--------- START DIFFIE HELLMAN KEY EXCHANGE TEST ---------<br /><br />\n";
            self::bcmath_diffieHellman($verbose);
            echo "<br /><br />--------- END DIFFIE HELLMAN KEY EXHANGE TEST ---------<br /><br />\n";
            $end_time = microtime(true);

            $time_res = $end_time - $start_time;

            echo "<br /><h3>TEST SUITE TOTAL TIME : " . $time_res . " seconds. </h3><br />";
        } else {
            echo "Please install GMP or BCMATH. For higher performance GMP is preferred.";
        }
    }



 
    public static function test_signature_validity($Msg, $Qx, $Qy, $R, $S, $expected, $verbose = false) {
        $p192 = NISTcurve::generator_192();
        $curve_192 = NISTcurve::curve_192();

        $pubk = new PublicKey($p192, new Point($curve_192, $Qx, $Qy));
        $got = $pubk->verifies(PrivateKey::digest_integer($Msg), new Signature($R, $S));
        if (bccomp($got, $expected) == 0) {
            if ($verbose)
                print "Signature tested as expected: received " . var_export($got, true) . ", expected " . var_export($expected, true) . ".<br />";
            flush();
        }else
            print "*** Signature test failed: received " . var_export($got, true) . ", expected " . var_export($expected, true) . ".<br />";
        flush();
    }

}
?>
